Imports Microsoft.VisualBasic
Imports System

Namespace ParticleSwarmOptimization
    Friend Class Program
        Private Shared ran As Random = Nothing
        Shared Sub Main(ByVal args() As String)
            Try
                Console.WriteLine(vbLf & "Begin Particle Swarm Optimization demonstration" & vbLf)
                Console.WriteLine(vbLf & "Objective function to minimize has dimension = 2")
                Console.WriteLine("Objective function is f(x) = 3 + (x0^2 + x1^2)")

                ran = New Random(0)

                Dim numberParticles As Integer = 10
                Dim numberIterations As Integer = 1000
                Dim iteration As Integer = 0
                ' dimensions
                Dim [Dim] As Integer = 2
                Dim minX As Double = -100.0
                Dim maxX As Double = 100.0

                Console.WriteLine("Range for all x values is " & minX & " <= x <= " & maxX)
                Console.WriteLine(vbLf & "Number iterations = " & numberIterations)
                Console.WriteLine("Number particles in swarm = " & numberParticles)

                Dim swarm(numberParticles - 1) As Particle
                ' best solution found by any particle in the swarm. implicit initialization to all 0.0
                Dim bestGlobalPosition([Dim] - 1) As Double
                ' smaller values better
                Dim bestGlobalFitness As Double = Double.MaxValue

                Dim minV As Double = -1.0 * maxX
                Dim maxV As Double = maxX

                Console.WriteLine(vbLf & "Initializing swarm with random positions/solutions")
                ' initialize each Particle in the swarm
                For i As Integer = 0 To swarm.Length - 1
                    Dim randomPosition([Dim] - 1) As Double
                    For j As Integer = 0 To randomPosition.Length - 1
                        Dim lo As Double = minX
                        Dim hi As Double = maxX
                        randomPosition(j) = (hi - lo) * ran.NextDouble() + lo
                    Next j
                    ' double fitness = SphereFunction(randomPosition); // smaller values are better
                    ' double fitness = GP(randomPosition); // smaller values are better
                    Dim fitness As Double = ObjectiveFunction(randomPosition)
                    Dim randomVelocity([Dim] - 1) As Double

                    For j As Integer = 0 To randomVelocity.Length - 1
                        Dim lo As Double = -1.0 * Math.Abs(maxX - minX)
                        Dim hi As Double = Math.Abs(maxX - minX)
                        randomVelocity(j) = (hi - lo) * ran.NextDouble() + lo
                    Next j
                    swarm(i) = New Particle(randomPosition, fitness, randomVelocity, randomPosition, fitness)

                    ' does current Particle have global best position/solution?
                    If swarm(i).fitness < bestGlobalFitness Then
                        bestGlobalFitness = swarm(i).fitness
                        swarm(i).position.CopyTo(bestGlobalPosition, 0)
                    End If
                    ' initialization
                Next i

                Console.WriteLine(vbLf & "Initialization complete")
                Console.WriteLine("Initial best fitness = " & bestGlobalFitness.ToString("F4"))
                Console.WriteLine("Best initial position/solution:")
                For i As Integer = 0 To bestGlobalPosition.Length - 1
                    Console.WriteLine("x" & i & " = " & bestGlobalPosition(i).ToString("F4") & " ")
                Next i

                ' inertia weight. see http://ieeexplore.ieee.org/stamp/stamp.jsp?arnumber=00870279
                Dim w As Double = 0.729
                ' cognitive/local weight
                Dim c1 As Double = 1.49445
                ' social/global weight
                Dim c2 As Double = 1.49445
                ' cognitive and social randomizations
                Dim r1, r2 As Double

                Console.WriteLine(vbLf & "Entering main PSO processing loop")
                Do While iteration < numberIterations
                    iteration += 1
                    Dim newVelocity([Dim] - 1) As Double
                    Dim newPosition([Dim] - 1) As Double
                    Dim newFitness As Double

                    ' each Particle
                    For i As Integer = 0 To swarm.Length - 1
                        Dim currP As Particle = swarm(i)

                        ' each x value of the velocity
                        For j As Integer = 0 To currP.velocity.Length - 1
                            r1 = ran.NextDouble()
                            r2 = ran.NextDouble()

                            newVelocity(j) = (w * currP.velocity(j)) + (c1 * r1 * (currP.bestPosition(j) - currP.position(j))) + (c2 * r2 * (bestGlobalPosition(j) - currP.position(j)))

                            If newVelocity(j) < minV Then
                                newVelocity(j) = minV
                            ElseIf newVelocity(j) > maxV Then
                                newVelocity(j) = maxV
                            End If
                        Next j

                        newVelocity.CopyTo(currP.velocity, 0)

                        For j As Integer = 0 To currP.position.Length - 1
                            newPosition(j) = currP.position(j) + newVelocity(j)
                            If newPosition(j) < minX Then
                                newPosition(j) = minX
                            ElseIf newPosition(j) > maxX Then
                                newPosition(j) = maxX
                            End If
                        Next j

                        newPosition.CopyTo(currP.position, 0)
                        newFitness = ObjectiveFunction(newPosition)
                        currP.fitness = newFitness

                        If newFitness < currP.bestFitness Then
                            newPosition.CopyTo(currP.bestPosition, 0)
                            currP.bestFitness = newFitness
                        End If

                        If newFitness < bestGlobalFitness Then
                            newPosition.CopyTo(bestGlobalPosition, 0)
                            bestGlobalFitness = newFitness
                        End If

                        ' each Particle
                    Next i

                    Console.WriteLine(swarm(0).ToString())
                    Console.ReadLine()

                    ' while
                Loop

                Console.WriteLine(vbLf & "Processing complete")
                Console.Write("Final best fitness = ")
                Console.WriteLine(bestGlobalFitness.ToString("F4"))
                Console.WriteLine("Best position/solution:")
                For i As Integer = 0 To bestGlobalPosition.Length - 1
                    Console.Write("x" & i & " = ")
                    Console.WriteLine(bestGlobalPosition(i).ToString("F4") & " ")
                Next i
                Console.WriteLine("")

                Console.WriteLine(vbLf & "End PSO demonstration" & vbLf)
                Console.ReadLine()
            Catch ex As Exception
                Console.WriteLine("Fatal error: " & ex.Message)
                Console.ReadLine()
            End Try
            ' Main()
        End Sub

        Private Shared Function ObjectiveFunction(ByVal x() As Double) As Double
            ' f(x) = 3 + x^2 + y^2
            Return 3.0 + (x(0) * x(0)) + (x(1) * x(1))
        End Function
        ' class Program
    End Class

    Public Class Particle
        ' equivalent to x-Values and/or solution
        Public position() As Double
        Public fitness As Double
        Public velocity() As Double

        ' best position found so far by this Particle
        Public bestPosition() As Double
        Public bestFitness As Double

        Public Sub New(ByVal position() As Double, ByVal fitness As Double, ByVal velocity() As Double, ByVal bestPosition() As Double, ByVal bestFitness As Double)
            Me.position = New Double(position.Length - 1) {}
            position.CopyTo(Me.position, 0)
            Me.fitness = fitness
            Me.velocity = New Double(velocity.Length - 1) {}
            velocity.CopyTo(Me.velocity, 0)
            Me.bestPosition = New Double(bestPosition.Length - 1) {}
            bestPosition.CopyTo(Me.bestPosition, 0)
            Me.bestFitness = bestFitness
        End Sub

        Public Overrides Function ToString() As String
            Dim s As String = ""
            s &= "==========================" & vbLf
            s &= "Position: "
            For i As Integer = 0 To Me.position.Length - 1
                s &= Me.position(i).ToString("F2") & " "
            Next i
            s &= vbLf
            s &= "Fitness = " & Me.fitness.ToString("F4") & vbLf
            s &= "Velocity: "
            For i As Integer = 0 To Me.velocity.Length - 1
                s &= Me.velocity(i).ToString("F2") & " "
            Next i
            s &= vbLf
            s &= "Best Position: "
            For i As Integer = 0 To Me.bestPosition.Length - 1
                s &= Me.bestPosition(i).ToString("F2") & " "
            Next i
            s &= vbLf
            s &= "Best Fitness = " & Me.bestFitness.ToString("F4") & vbLf
            s &= "==========================" & vbLf
            Return s
        End Function

        ' class Particle
    End Class

    ' ns
End Namespace